package org.apfloat.internal;

import org.apfloat.*;
import org.apfloat.spi.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @version 1.0.1
 * @author Mikko Tommila
 */

public class Long3NTTConvolutionStrategyTest
    extends LongConvolutionStrategyTestCase
    implements LongRadixConstants
{
    public Long3NTTConvolutionStrategyTest(String methodName)
    {
        super(methodName);
    }

    public static void main(String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }

    public static TestSuite suite()
    {
        TestSuite suite = new TestSuite();

        suite.addTest(new Long3NTTConvolutionStrategyTest("testFull"));
        suite.addTest(new Long3NTTConvolutionStrategyTest("testTruncated"));
        suite.addTest(new Long3NTTConvolutionStrategyTest("testAuto"));
        suite.addTest(new Long3NTTConvolutionStrategyTest("testFullBig"));
        suite.addTest(new Long3NTTConvolutionStrategyTest("testTruncatedBig"));
        suite.addTest(new Long3NTTConvolutionStrategyTest("testAutoBig"));

        return suite;
    }

    public static void testFull()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            long b1 = BASE[radix] - (long) 1;
            DataStorage src1 = createDataStorage(new long[] { (long) 1, (long) 2, (long) 3, (long) 4 }),
                        src2 = createDataStorage(new long[] { (long) 5, (long) 6 }),
                        src9 = createDataStorage(new long[] { b1, b1, b1 }),
                        src99 = createDataStorage(new long[] { b1, b1, b1, b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Long3NTTConvolutionStrategy(radix, new LongTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src1, src2, 6);

            check("normal", radix, new long[] { 0, (long) 5, (long) 16, (long) 27, (long) 38, (long) 24 }, result);

            result = convolutionStrategy.convolute(src9, src99, 9);

            check("max", radix, new long[] { b1, b1, b1 - (long) 1, b1, b1, b1, 0, 0, (long) 1 }, result);
        }
    }

    public static void testTruncated()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            long b1 = BASE[radix] - (long) 1;
            DataStorage src1 = createDataStorage(new long[] { (long) 1, (long) 2, (long) 3, (long) 4 }),
                        src2 = createDataStorage(new long[] { (long) 5, (long) 6 }),
                        src9 = createDataStorage(new long[] { b1, b1, b1 }),
                        src99 = createDataStorage(new long[] { b1, b1, b1, b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Long3NTTConvolutionStrategy(radix, new LongTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src1, src2, 3);

            check("normal", radix, new long[] { 0, (long) 5, (long) 16 }, result);

            result = convolutionStrategy.convolute(src9, src99, 6);

            check("max", radix, new long[] { b1, b1, b1 - (long) 1, b1, b1, b1 }, result);
        }
    }

    public static void testAuto()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            long b1 = BASE[radix] - (long) 1;
            DataStorage src9 = createDataStorage(new long[] { b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Long3NTTConvolutionStrategy(radix, new LongTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src9, 6);

            check("max", radix, new long[] { b1, b1, b1 - (long) 1, 0, 0, (long) 1 }, result);
        }
    }

    public static void testFullBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            long b1 = BASE[radix] - (long) 1;
            int size1 = 500,
                size2 = 300;
            long[] array1 = new long[size1],
                      array2 = new long[size2],
                      array3 = new long[size1 + size2];
            for (int i = 0; i < size1; i++)
            {
                array1[i] = b1;
                array3[i] = b1 - (long) (i == size2 - 1 ? 1 : 0);
            }
            for (int i = 0; i < size2; i++)
            {
                array2[i] = b1;
                array3[i + size1] = (long) (i == size2 - 1 ? 1 : 0);
            }
            DataStorage src9 = createDataStorage(array1),
                        src99 = createDataStorage(array2);

            ConvolutionStrategy convolutionStrategy = new Long3NTTConvolutionStrategy(radix, new LongTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src99, size1 + size2);

            check("max", radix, array3, result);
        }
    }

    public static void testTruncatedBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            long b1 = BASE[radix] - (long) 1;
            int size1 = 500,
                size2 = 300;
            long[] array1 = new long[size1],
                      array2 = new long[size2],
                      array3 = new long[size1];
            for (int i = 0; i < size1; i++)
            {
                array1[i] = b1;
                array3[i] = b1 - (long) (i == size1 - 1 || i == size2 - 1 ? 1 : 0);
            }
            for (int i = 0; i < size2; i++)
            {
                array2[i] = b1;
            }
            DataStorage src9 = createDataStorage(array1),
                        src99 = createDataStorage(array2);

            ConvolutionStrategy convolutionStrategy = new Long3NTTConvolutionStrategy(radix, new LongTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src99, src9, size1);

            check("max", radix, array3, result);
        }
    }

    public static void testAutoBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            long b1 = BASE[radix] - (long) 1;
            int size = 500;
            long[] array1 = new long[size],
                      array2 = new long[2 * size];
            for (int i = 0; i < size; i++)
            {
                array1[i] = b1;
                array2[i] = b1 - (long) (i == size - 1 ? 1 : 0);
                array2[i + size] = (long) (i == size - 1 ? 1 : 0);
            }
            DataStorage src9 = createDataStorage(array1);

            ConvolutionStrategy convolutionStrategy = new Long3NTTConvolutionStrategy(radix, new LongTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src9, 2 * size);

            check("max", radix, array2, result);
        }
    }
}
